package net.lamgc.scext.onedrive_transfer

import com.microsoft.aad.msal4j.ITokenCacheAccessAspect
import com.microsoft.aad.msal4j.ITokenCacheAccessContext
import mu.KotlinLogging
import org.jetbrains.exposed.dao.LongEntity
import org.jetbrains.exposed.dao.LongEntityClass
import org.jetbrains.exposed.dao.id.EntityID
import org.jetbrains.exposed.dao.id.LongIdTable
import org.jetbrains.exposed.sql.Database
import org.jetbrains.exposed.sql.transactions.transaction

object OneDriveTransferSettings : LongIdTable() {
    val telegramUserId = long("tg_user_id").uniqueIndex()
    val accountId = varchar("account_id", 128)
    val userName = varchar("user_name", 96)
    val driveId = varchar("drive_id", 256)
    val storagePath = varchar("storage_path", 512)
}

class OneDriveTransferSetting(id: EntityID<Long>) : LongEntity(id) {
    var telegramUserId by OneDriveTransferSettings.telegramUserId
    var accountId by OneDriveTransferSettings.accountId
    var userName by OneDriveTransferSettings.userName
    var driveId by OneDriveTransferSettings.driveId
    var storagePath by OneDriveTransferSettings.storagePath

    companion object : LongEntityClass<OneDriveTransferSetting>(OneDriveTransferSettings)
}

object TokenCaches : LongIdTable() {
    val clientId = varchar("client_id", 256).uniqueIndex()
    val cache = text("cache_data")
}

class TokenCache(id: EntityID<Long>) : LongEntity(id) {
    var clientId by TokenCaches.clientId
    var cache by TokenCaches.cache

    companion object : LongEntityClass<TokenCache>(TokenCaches)
}

class DatabaseTokenCache(private val db: Database) : ITokenCacheAccessAspect {

    override fun beforeCacheAccess(context: ITokenCacheAccessContext) {
        transaction(db) {
            TokenCache.find { TokenCaches.clientId eq context.clientId() }.firstOrNull()?.let {
                context.tokenCache().deserialize(it.cache)
            }
        }
    }

    override fun afterCacheAccess(context: ITokenCacheAccessContext) {
        if (!context.hasCacheChanged()) {
            return
        }
        transaction(db) {
            val existCache =
                TokenCache.find { TokenCaches.clientId eq context.clientId() }.firstOrNull()
            if (existCache == null) {
                TokenCache.new {
                    clientId = context.clientId()
                    cache = context.tokenCache().serialize()
                }
            } else {
                existCache.cache = context.tokenCache().serialize()
            }
        }
    }
}
